package models

import (
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"hongze/hongze_edb_lib/utils"
	"sort"
	"time"
)

// EdbDataCalculateLjztbpj 累计值同比拼接数据结构体
type EdbDataCalculateLjztbpj struct {
	EdbDataId     int `orm:"column(edb_data_id);pk"`
	EdbInfoId     int
	EdbCode       string
	DataTime      string
	Value         float64
	Status        int
	CreateTime    time.Time
	ModifyTime    time.Time
	DataTimestamp int64
}

// AddCalculateLjztbpj 新增累计值同比拼接数据
func AddCalculateLjztbpj(req *EdbInfoCalculateBatchSaveReq, firstEdbInfo, secondEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string) (edbInfoId int, err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("AddCalculateLjztbpj,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	//待拼接指标map
	pjDataMap := make(map[string]float64)

	//拼接指标的日期切片数据
	pjEdbDataTimeList := make([]string, 0)

	//最近开始的时间
	var lastDateTime time.Time
	//获取待拼接指标
	{
		var condition string
		var pars []interface{}

		//获取待拼接指标最近的个12月31日有值的年份
		condition += " AND data_time like ? AND edb_info_id=? "
		pars = append(pars, "%12-31", firstEdbInfo.EdbInfoId)

		lastEdbData, tmpErr := GetLastEdbData(condition, pars, firstEdbInfo.Source)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}
		lastDateTime, _ = time.ParseInLocation(utils.FormatDate, lastEdbData.DataTime, time.Local)

		//获取待拼接指标的数据列表
		condition = ``
		pars = make([]interface{}, 0)
		condition += " AND data_time <= ? AND edb_info_id=? "
		pars = append(pars, lastEdbData.DataTime, firstEdbInfo.EdbInfoId)

		firstDataList, tmpErr := GetEdbDataListAll(condition, pars, firstEdbInfo.Source, 0)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}

		for _, v := range firstDataList {
			pjDataMap[v.DataTime] = v.Value
			pjEdbDataTimeList = append(pjEdbDataTimeList, v.DataTime)
		}
	}

	var edbInfo *EdbInfo
	if req.EdbInfoId <= 0 {
		edbInfo = &EdbInfo{
			SourceName:       "累计值同比拼接",
			Source:           utils.DATA_SOURCE_CALCULATE_LJZTBPJ,
			EdbCode:          edbCode,
			EdbName:          req.EdbName,
			EdbNameSource:    req.EdbName,
			Frequency:        req.Frequency,
			Unit:             req.Unit,
			StartDate:        firstEdbInfo.StartDate,
			EndDate:          firstEdbInfo.EndDate,
			ClassifyId:       req.ClassifyId,
			SysUserId:        sysUserId,
			SysUserRealName:  sysUserRealName,
			UniqueCode:       uniqueCode,
			CreateTime:       time.Now(),
			ModifyTime:       time.Now(),
			CalculateFormula: lastDateTime.Format(utils.FormatDate),
			EdbType:          2,
		}
		newEdbInfoId, err := to.Insert(edbInfo)
		if err != nil {
			return edbInfoId, err
		}
		edbInfoId = int(newEdbInfoId)
	} else {
		edbInfoId = req.EdbInfoId
		//查询
		tmpEdbInfo, tmpErr := GetEdbInfoById(edbInfoId)
		if tmpErr != nil {
			err = tmpErr
			return
		}
		tmpEdbInfo.EdbName = req.EdbName
		tmpEdbInfo.ClassifyId = req.ClassifyId
		tmpEdbInfo.Frequency = req.Frequency
		tmpEdbInfo.Unit = req.Unit
		tmpEdbInfo.CalculateFormula = req.Formula

		edbInfo = tmpEdbInfo

		//删除指标数据
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_LJZTBPJ)
		fmt.Println("dataTableName:" + dataTableName)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
		if err != nil {
			return 0, err
		}

		//删除指标关系
		sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id=? `
		_, err = to.Raw(sql, edbInfoId).Exec()
	}

	//关联关系

	//拼接指标
	{
		calculateMappingItem := new(EdbInfoCalculateMapping)
		calculateMappingItem.CreateTime = time.Now()
		calculateMappingItem.ModifyTime = time.Now()
		calculateMappingItem.Sort = 1
		calculateMappingItem.EdbCode = edbCode
		calculateMappingItem.EdbInfoId = edbInfoId
		calculateMappingItem.FromEdbInfoId = firstEdbInfo.EdbInfoId
		calculateMappingItem.FromEdbCode = firstEdbInfo.EdbCode
		calculateMappingItem.FromEdbName = firstEdbInfo.EdbName
		calculateMappingItem.FromSource = firstEdbInfo.Source
		calculateMappingItem.FromSourceName = firstEdbInfo.SourceName
		calculateMappingItem.FromTag = "A"
		calculateMappingItem.Source = edbInfo.Source
		calculateMappingItem.SourceName = edbInfo.SourceName
		_, err = to.Insert(calculateMappingItem)
		if err != nil {
			return
		}
	}

	//同比值指标
	{
		calculateMappingItem := new(EdbInfoCalculateMapping)
		calculateMappingItem.CreateTime = time.Now()
		calculateMappingItem.ModifyTime = time.Now()
		calculateMappingItem.Sort = 1
		calculateMappingItem.EdbCode = edbCode
		calculateMappingItem.EdbInfoId = edbInfoId
		calculateMappingItem.FromEdbInfoId = secondEdbInfo.EdbInfoId
		calculateMappingItem.FromEdbCode = secondEdbInfo.EdbCode
		calculateMappingItem.FromEdbName = secondEdbInfo.EdbName
		calculateMappingItem.FromSource = secondEdbInfo.Source
		calculateMappingItem.FromSourceName = secondEdbInfo.SourceName
		calculateMappingItem.FromTag = "B"
		calculateMappingItem.Source = edbInfo.Source
		calculateMappingItem.SourceName = edbInfo.SourceName
		_, err = to.Insert(calculateMappingItem)
		if err != nil {
			return
		}
	}

	//同比值指标map
	tbzEdbDataMap := make(map[string]float64)

	//同比值日期切片列表
	tbzEdbDataTimeList := make([]string, 0)

	//同比值指标
	{
		var condition string
		var pars []interface{}

		condition += " AND data_time > ? AND edb_info_id = ? "
		pars = append(pars, lastDateTime, secondEdbInfo.EdbInfoId)

		//同比值指标的数据列表
		secondDataList, tmpErr := GetEdbDataListAll(condition, pars, secondEdbInfo.Source, 0)
		if tmpErr != nil {
			return edbInfoId, tmpErr
		}

		for _, v := range secondDataList {
			tbzEdbDataMap[v.DataTime] = v.Value
			tbzEdbDataTimeList = append(tbzEdbDataTimeList, v.DataTime)
		}
	}

	sort.Strings(tbzEdbDataTimeList)
	for _, v := range tbzEdbDataTimeList {
		tbzDataTime, _ := time.ParseInLocation(utils.FormatDate, v, time.Local)

		//获取拼接指标上一年同一天的数据
		var pjDataTime time.Time
		if tbzDataTime.Month() == 2 {
			pjDataTime = tbzDataTime.AddDate(0, -11, 0)
			pjDataTime = time.Date(pjDataTime.Year(), pjDataTime.Month(), 1, 0, 0, 0, 0, time.Now().Location()).AddDate(0, 0, -1)
		} else {
			pjDataTime = tbzDataTime.AddDate(-1, 0, 0)
		}

		//如果存在数据，那么就去计算当前的数据
		if pjDataValue, ok := pjDataMap[pjDataTime.Format(utils.FormatDate)]; ok {
			tbzDataValue := tbzEdbDataMap[v] //同比值
			currValue := pjDataValue * (1 + tbzDataValue/100)

			currValue, _ = decimal.NewFromFloat(currValue).Truncate(4).Float64()
			//将计算后的数据存入待拼接指标map里面，以便后续计算
			pjDataMap[v] = currValue

			pjEdbDataTimeList = append(pjEdbDataTimeList, v)
		}
	}

	addDataList := make([]*EdbDataCalculateLjztbpj, 0)

	//日期排序下
	sort.Strings(pjEdbDataTimeList)
	//这么做的目的是为了让数据插入的时候，可以正序插入（业务上没啥卵用，就是为了让我看数据的时候舒服点，手动狗头-_-|）
	for _, dataTime := range pjEdbDataTimeList {
		if dataValue, ok := pjDataMap[dataTime]; ok {
			//时间戳
			currentDate, _ := time.Parse(utils.FormatDate, dataTime)
			timestamp := currentDate.UnixNano() / 1e6

			edbDataLjztbpj := &EdbDataCalculateLjztbpj{
				EdbInfoId:     edbInfoId,
				EdbCode:       edbInfo.EdbCode,
				DataTime:      dataTime,
				Value:         dataValue,
				Status:        1,
				CreateTime:    time.Now(),
				ModifyTime:    time.Now(),
				DataTimestamp: timestamp,
			}
			addDataList = append(addDataList, edbDataLjztbpj)
		}
	}

	//数据入库
	tmpAddDataList := make([]*EdbDataCalculateLjztbpj, 0)
	for _, v := range addDataList {
		tmpAddDataList = append(tmpAddDataList, v)

		if len(tmpAddDataList) >= 200 {
			_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
			if tmpErr != nil {
				err = tmpErr
				return
			}
			//重新初始化需要加入的数据切片
			tmpAddDataList = make([]*EdbDataCalculateLjztbpj, 0)
		}
	}
	//最后如果还有需要新增的数据，那么就统一入库
	if len(tmpAddDataList) > 0 {
		_, tmpErr := to.InsertMulti(len(tmpAddDataList), tmpAddDataList)
		if tmpErr != nil {
			err = tmpErr
			return
		}
	}

	return
}
